#!/usr/bin/perl
# index.cgi

$no_acl_check = 1;

require './batch-dns-lib.pl';

my (@new_domains, @bad_domains);
my %bad_reason;
my $address;
my $directives;
my $ip;

&ReadParse;
&process;
&output;

sub process {
    # modifies globals: @bad_domains, %bad_reason, @new_domains, $ip.

    foreign_require("bind8", "bind8-lib.pl");

    my $error;
    my $rawlist = $in{'domains'};
    my @raw_domains = split '\n', $rawlist;
    my $named_conf_additions;
    my $acl_additions;

    unless (&check_ipaddress($in{'ip'}))
    {
	&check_ipaddresses(to_ipaddress($in{'ip'}))
	    || &error("Bad name or IP address for these domains.");
    }
    
    $ip = $in{'ip'};
    my $master_ip = to_ipaddress($config{'master'}) || 
	&error("Master has bad IP address.");

    # required configuration parameters
    
    $config{'master'} =~ /^[A-z0-9\-\.]+$/ ||
        &error(&text('master_emaster', $config{'master'}));
    
    if ($config{'master'} !~ /\.$/) { $config{'master'} .= "."; }
    if ($config{'email_def'}) {
	$config{'email'} =~ /^\S+$/ ||
	    &error(&text('master_eemail', $config{'email'}));
	if ($config{'email'} =~ s/\@/\./) { $config{'email'} .= "."; }
    }
    
    $config{'refresh'} =~ /^\S+$/ ||
        &error(&text('master_erefresh', $config{'refresh'}));
    $config{'retry'} =~ /^\S+$/ ||
        &error(&text('master_eretry', $config{'retry'}));
    $config{'expiry'} =~ /^\S+$/ ||
        &error(&text('master_eexpiry', $config{'expiry'}));
    $config{'minimum'} =~ /^\S+$/ ||
        &error(&text('master_eminimum', $config{'minimum'}));

    $conf = foreign_call('bind8', 'get_config');    
    $base = foreign_call('bind8', 'base_directory', $conf);
    
    %bind8_config = foreign_config('bind8');

    # name must have one or more '.', no illegal characters, no repeats.
    foreach $domain (@raw_domains) {
	$domain =~ s/\s$//;
	if ($domain =~ /^\s*$/) { # entirely whitespace
	    next;
	} elsif ($domain !~ /^[A-z0-9\-\.]+$/ ) {
	    push @bad_domains, $domain;
	    $bad_reason{$domain} = &text('mcreate_edom', $domain);
	} elsif ($domain =~ /^[0-9\.]+$/ ) {
	    push @bad_domains, $domain;
	    $bad_reason{$domain} = &text('mcreate_edom2', $domain);
        } else {
	    $domain =~ s/\.$//g;
	    push @new_domains, $domain;
	}
    }

    if ($in{'secondary'}) {
	# slave server

	my $fullpath;
	my $email = $config{'email'};
	foreach $domain (@new_domains) {
	    $fullpath = "$base/$domain.hosts";
	    $named_conf_additions .= <<EOF;
zone "$domain" {
        type slave;
	file "$fullpath";
	masters { $master_ip; };
};
EOF
}
    } else {
	# master server

	my $fullpath;
	my $email = $config{'email'};
	foreach $domain (@new_domains) {
	    my $vals;
	    
	    $fullpath = "$base/$domain.hosts";
	    unless ($config{'email_def'}) {
		$email = "root.$domain";
	    }
	    
	    $named_conf_additions .= <<EOF;
zone "$domain" {
         type master;
         file "$fullpath";
};

EOF
            # create the SOA and NS records
            if ($bind8_config{'soa_style'} == 1) {
		$serial = foreign_call('bind8','date_serial')."00";
	    }
	    
	    else {
		$serial = time();
	    }
	    $vals = "$config{'master'} $email (\n".
		"\t\t\t$serial\n".
		"\t\t\t$config{'refresh'}\n".
	        "\t\t\t$config{'retry'}\n".
     	        "\t\t\t$config{'expiry'}\n".
	        "\t\t\t$config{'minimum'} )";
	    foreign_call('bind8','create_record',$fullpath, "$domain.", undef, 
			 "IN", "SOA", $vals);
	    foreign_call('bind8','create_record',$fullpath, undef, undef, 
			 undef, "NS", $config{'master'});
	    foreign_call('bind8','create_record',$fullpath, undef, undef, 
			 undef, "A", $ip);
	    foreign_call('bind8','create_record',$fullpath, "*", undef, 
			 undef, "A", $ip);
	    
	    $acl_additions .= " $domain";
	}

	# Add the new zone to the access list
	if ($access{'zones'} ne '*') {
	    $access{'zones'} .= " ".$acl_additions;
	    &save_module_acl(\%access);
	}
    }
	
    # create the zone directive
    open(CONF, ">> $bind8_config{'named_conf'}");
    print CONF $named_conf_additions;
    close(CONF);
}
	   
sub output {

    my $domain;
    my $type = $in{'secondary'} ? "Slave" : "Master";

    &header("Adding DNS $type Domains", "", "");
    if (@bad_domains) {
	print "Errors with the following ", int(@bad_domains), " domain(s):<P>\n";
	print "<table>\n";
	print "<tr><td><strong>Domain Name &nbsp;&nbsp;&nbsp;</strong></td>";
	print "<td><strong> Reason</strong></td></tr>";
	foreach $domain (@bad_domains) {
	    print "<tr><td>$domain</td><td>$bad_reason{$domain}</td></tr>\n";
	}
	print "</table>\n";
	print "<hr>\n";
    }
    if (@new_domains) {
	print "Added the following ", int(@new_domains), " $type domain(s):<P>\n";
	print "<table>\n";
	print "<tr><td><strong>Domain Name &nbsp;&nbsp;&nbsp;</strong></td></tr>";
	foreach $domain (@new_domains) {
	    print "<tr><td>$domain</td></tr>\n";
	}
	print "</table>\n";
    
	print "<hr>\n";
    } else {
	print "Did not add any new domains.\n";
	print "<hr>\n";
    }

    # get pid
    $pidfile = "/var/run/named.pid";
    
    if (open(PID, $pidfile)) {
        <PID> =~ /(\d+)/;
        $pid = $1;
        close(PID);
    }

    if ($pid && kill(0, $pid)) {
	print "Restarting DNS server...<P>\n";
        # named is running
	kill('HUP', $pid) || &error(&text('restart_esig', $pid, $!));
    }

    # chain to another server, if necessary.

    my $out_rawlist = join "\n", @new_domains;

    if ($config{'follow'} and (!($config{'secondary'}))) {
	print "<form action=\"$config{'follow'}\" method=\"post\">\n";
	print "<input type=\"hidden\" name=\"secondary\" value=\"1\">\n";
	print "<input type=\"hidden\" name=\"ip\" value=\"$ip\">\n";
	print "<input type=\"hidden\" name=\"domains\" value=\"$out_rawlist\">\n";
	print "<input type=\"submit\" value=\"Add Slave DNS\"><P>\n";
    }

    &footer("/", "index");
    
}
