#!/usr/bin/perl
#
# $Id: openldap-lib.pl,v 1.7 2000/12/20 09:08:31 seb Exp $
# Common routines
#
#    OpenLDAP Webmin Module
#    Copyright (C) 2000 by S. Namche <snameche@virtual-net.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#

require './parse_slapd_conf.pl';
do '../web-lib.pl';
init_config();
ReadParse();

sub print_header {
    my( $title, $file, $type, $suffix ) = @_;
    header( $text{index_title}, undef, undef, 1, undef, undef, $text{index_right} );
    print "<center><b>$title ";
    print $type eq $GLOBAL_TYPE ? $text{global_options} : "backend \"$suffix\" ($type)";
    print $file eq $config{slapd_conf_file} ? '</b>' : "</b><br>$text{include_file} $file";
    print "</center><hr>\n";
}

sub get_backends {
    my( $b, $i ) = @_;

    my @backends = parse_slapd_conf( $config{slapd_conf_file} );
    return unless @backends;
    my $type     = $backends[$b]{type};
    my $suffix   = $backends[$b]{suffix};
    
    if( $i ){
	my $file = $backends[$b]{include}[$i-1];
	my @tmp = parse_slapd_conf( $file );
	return( $file, $type, $suffix, \@tmp );
    }

    return( $config{slapd_conf_file}, $type, $suffix, \@backends );
}

sub can_edit_backend {
    my( $b, $i ) = @_;
    return 1;
}

# DN_chooser_button(field, multiple, base, [host], [port], [filter], [form])
# Returns HTML for a javascript button for choosing one or a list of LDAP DN
sub DN_chooser_button {
    my $field  = shift;
    my $multi  = shift;
    my $base   = canonic_dn( shift );
    my $host   = shift || 'localhost';
    my $port   = shift || 389;
    my $filter = shift || 'objectclass=*';
    my $form   = shift || 0;

    my $script = << "EOS";
ifield = document.forms[$form].$field;
chooser = window.open( "/DN_chooser.cgi?multi=$multi" +
		           "&user="   + escape(ifield.value) +
		           "&base="   + escape("$base") +
                           "&host="   + escape("$host") +
                           "&port="   + escape("$port") +
		           "&filter=" + escape("$filter"),
		       "chooser",
		       "toolbar=no,menubar=no,scrollbars=yes,width=600,height=300" );
chooser.ifield = ifield;
EOS

    return "<input type=button onClick='$script' value=\"DN\">\n";
}

sub yesno_input {
    my( $name, $value, $yesno_only ) = @_;

    if( $yesno_only ) {
	return sprintf "<input type=radio name=$name value=1 %s> $text{yes}\n" .
	    "<input type=radio name=$name value=0 %s> $text{no}\n",
	    ($value eq "1") ? "checked" : "",
	    ($value eq "0") ? "checked" : "";
    }
    else {
	return sprintf "<input type=radio name=$name value=\"\" %s> $text{default}\n" .
	    "<input type=radio name=$name value=1 %s> $text{yes}\n" .
	    "<input type=radio name=$name value=0 %s> $text{no}\n",
	    ($value eq "")  ? "checked" : "",
	    ($value eq "1") ? "checked" : "",
	    ($value eq "0") ? "checked" : "";
    }
}

sub file_input {
    my( $name, $value ) = @_;

    return "<input size=40 name=$name value=\"$value\">\n".
	&file_chooser_button( $name );
}

sub list_input {
    my( $name, $labels, $values, $selected ) = @_;

    my $return = "<select name=\"$name\">\n";
    for my $i (0..$#{$labels}) {
	$return .= sprintf "<option value=\"$values->[$i]\" %s>$labels->[$i]\n",
	           $selected eq $values->[$i] ? "selected" : "";
    }

    return $return . "</select>\n";
}

sub canonic_dn {
    my $dn = shift;
    $dn =~ s/\s*"?([^"]+)"?\s*/$1/;
    $dn =~ s/^\s*//;
    $dn =~ s/\s*$//;
    $dn =~ s/\s*(,|=)\s*/$1/g;
    return $dn;
}

sub compare_dns {
    my $one = canonic_dn( $_[0] );
    my $two = canonic_dn( $_[1] );
    return ($one eq $two);
}

1;
