/*
 *    author: Lloyd Dupont
 *    mail: lloyd@galador.net
 *    Copyright (C) 2000 Lloyd Dupont
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.awt.*;
import java.util.*;
import java.net.*;

/**
 * a class to load image. it load image from a name. <U>It is not lazy</U>,
 * returning fully initialized image (unlikely other java method). It use
 * URLBuilder to build complete URL to image. <BR>
 * It maintain a table of already loaded/scaled image, so, each image are
 * loaded only one time.
 *
 * @see #loadImage
 * @see #flush
 * @see URLBuilder
 */
public class ImageCenter
{
    static Hashtable images = new Hashtable();

    public static Image loadImage(String aName, int x, int y) {
		return loadImage(URLBuilder.find(aName), x, y);
    }

    public static Image loadImage(String name) {
		return loadImage(URLBuilder.find(name));
    }

    public static Image loadImage(URL u) {
        Image i = (Image) images.get(u);
		if(i != null)
			return i;
		i = Toolkit.getDefaultToolkit().getImage(u);
		flush(i);
		if(i == null)
			return null;
		images.put(u, i);
		return i;
	}
    public static Image loadImage(URL u, int x, int y) {
		Object key = new SizedKey(u, x, y);
        Image i = (Image) images.get(key);
		if(i != null)
			return i;
        i = loadImage(u);
        if(i == null)
            return null;
        i = i.getScaledInstance(x, y, Image.SCALE_SMOOTH);
        flush(i);
		images.put(key, i);
		return i;
    }
    private static Frame trackerFrame = new Frame("MediaTracker's InvisibleFrame");
    /**
    * get rid of this stupid, boring, shitting, asynchronous load
	* of image. load it. i am great enough to write thread when i need.
    */
    public static void flush(Image im) {
        if(im == null)
            return;
        MediaTracker track = new MediaTracker(trackerFrame);
        track.addImage(im, 0);
        try { track.waitForID(0); }
        catch(InterruptedException ie) { ie.printStackTrace(); }
    }

}
class SizedKey
{
	URL url;
	int width, height;

	SizedKey(URL u, int x, int y) {
		url = u;
		width = x;
		height = y;
	}
	public int hashCode() {
		return url.hashCode() ^ width ^ height;
	}
	public boolean equals(Object o) {
		if(o == this)
		    return true;
		if(o != null && o instanceof SizedKey) {
			SizedKey sk = (SizedKey) o;
			return url.equals(sk.url)
				&& (width == sk.width)
				&& (height == sk.height);
		}
		return false;
	}
}
