/*
 *    author: Lloyd Dupont
 *    mail: lloyd@galador.net
 *    Copyright (C) 2000 Lloyd Dupont
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;

/**
 * an Icon implementation, which display an image.
 */
public class ImageIcon implements Icon
{
    Image image;
    Dimension size;

    public ImageIcon(Image i) {
        ImageCenter.flush(i);
        image = i;
        size = new Dimension(image.getWidth(null), image.getHeight(null));
    }
    public ImageIcon(String s) {
        image = ImageCenter.loadImage(s);
        if(image == null)
            throw new NullPointerException(s + " - no such image");
        size = new Dimension(image.getWidth(null), image.getHeight(null));
    }
    public ImageIcon(URL u) {
        image = ImageCenter.loadImage(u);
        if(image == null)
            throw new NullPointerException(u + " - no such image");
        size = new Dimension(image.getWidth(null), image.getHeight(null));
    }
    /**
    * scale the image to exactly fit given dimension.
	* using Image.getScaledinstance method.
    */
    public ImageIcon(String s, int w, int h) {
        this(s, new Dimension(w, h));
    }
    /**
    * scale the image to exactly fit given dimension.
	* using Image.getScaledinstance method.
    */
    public ImageIcon(URL u, int w, int h) {
        this(u, new Dimension(w, h));
    }
    /**
    * make the image fit in the boundig box
    */
    public ImageIcon(String s, Dimension box) {
		this(URLBuilder.find(s), box);
    }
    /**
    * make the image fit in the boundig box
    */
    public ImageIcon(URL u, Dimension box) {
        this(u);
        if(box.width >= size.width && box.height >= size.height)
            return;
        if(box.width < 1 || box.height < 1)
            return;

        float scaleX = box.width / (float) size.width;
        float scaleY = box.height / (float) size.height;
        float scale = scaleX < scaleY ? scaleX : scaleY;

		if(scale > 1)
			return;
        image = ImageCenter.loadImage(u,
                                      (int) (size.width * scale),
                                      (int) (size.height * scale));

        size = new Dimension((int) (size.width * scale),
                             (int) (size.height * scale));
    }
    public Dimension getSize() { return size; }
    public void paint(Graphics g, int x, int y) { g.drawImage(image, x, y, null); }
}