/*
 *    author: Lloyd Dupont
 *    mail: lloyd@galador.net
 *    Copyright (C) 2000 Lloyd Dupont
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;

/**
 * the "commands" object in a menu. handle any single commands stuff, such as
 * click, display...
 */
class MenuCell
{
    static Icon arrow = new RightArrowIcon(5, 10);
    int inset = 1;
    Dimension size = getPreferredSize();
    Icon icon;
    String text = "Menu cell";
    MenuPanel parent;
    int startText = 0;
    Hashtable commands;
    boolean selected;

    public MenuCell() {}
    public MenuCell(MenuPanel aParent, Hashtable cmd)
    {
        parent = aParent;
        commands = cmd;
        setText((String) cmd.get("name"));

        Object o = cmd.get("icon");
        if(o instanceof String) {
            setIcon((String) o);
            cmd.put("icon", icon);
        }
        else
            setIcon((Icon) o);
    }

	public Hashtable getCommands() { return commands; }

    protected void setText(String s) {
			if("--".equals(s)) {
				text = "--";
				return;
			}
        text = s == null ? "" : s;
    }
    protected void setIcon(Icon anIcon) {
        icon = anIcon;
        if(icon != null)
            startText = icon.getSize().width + 2 * inset;
        else
            startText = 0;
    }
    public void setIcon(String name) {
		Applet a = parent.getController().applet;
		if(a != null)
			try {
				URL url = new URL(a.getDocumentBase(), name);
				setIcon(new ImageIcon(url, 50, 20));
			}
			catch(Exception ex) { setIcon((Icon) null); }
		else
			setIcon(name != null ? new ImageIcon(name, 50, 20) : null);
    }
    public boolean isDir() { return commands != null ? commands.get("child") != null : false; }

    public int getStartTextPosition() { return startText; }

    public void setSelected(boolean b)
    {
        MenuController mc = parent.getController();
        if(b && mc.applet != null)
            mc.applet.showStatus((String) commands.get("statut"));
        selected = b;
    }

    public boolean click() {
        MenuController mc = parent.getController();
		boolean somethingDone = false;

        if(mc.applet == null)
			return somethingDone;
		AppletContext ac = mc.applet.getAppletContext();
		if(ac == null)
			return somethingDone;

        String href       = (String) commands.get("href");
        String javascript = (String) commands.get("javascript");
        String applet     = (String) commands.get("applet");

        if(javascript != null) {
			parent.getController().todo.add(new JScriptAction(mc.applet, javascript));
			somethingDone = true;
		}

        if(applet != null) {
            Applet other  = ac.getApplet(applet);
			if(other != null) {
				try {
					Method m = other.getClass().getMethod("action", new Class[]{String.class});
	    			if(m != null)
		    			m.invoke(other, new Object[]{commands.get("action")});
					somethingDone = true;
				}
				catch(Throwable t) {
					System.err.println(t);
					t.printStackTrace();
				}
			}
        }

        if(href != null) {
            String target = (String) commands.get("target");
			URL url = null;
			try { url = new URL(mc.applet.getDocumentBase(), href); }
			catch(Exception ex) {}
            if(target != null)
                ac.showDocument(url, target);
            else
                ac.showDocument(url);
			somethingDone = true;
        }
		return somethingDone;
    }

    public Dimension getSize() { return size; }
    public void setSize(Dimension aSize) { size = aSize; }
    public Dimension getPreferredSize() {
        if(parent == null)
            return new Dimension(-1, -1);
        Graphics g = parent.getGraphics();
        if(g == null)
            return new Dimension(-1, -1);
        FontMetrics fm = g.getFontMetrics();
        if(fm == null)
            return new Dimension(-1, -1);

        int width = 0;
        if(isDir()) {
            Dimension size = arrow.getSize();
            width += size.width + 2 * inset;
        }
        width += 2 * inset + fm.stringWidth(text);
        int height = fm.getHeight() + 2 * inset;
		// 20 is icon y-size
        height = height < 20+2*inset ? 20+2*inset : height;

        return new Dimension(width + parent.getStartTextPosition(), height);
    }
    public void paint(Graphics g) {
        Dimension dim = getSize();
        if(selected) {
            Color c = g.getColor();
            g.setColor(parent.getController().selcolor);
            g.fillRect(0, 0, dim.width, dim.height);
            g.setColor(c);
        }

			if(text == "--") {
				g.drawLine(5+inset, dim.height/2, dim.width-inset-5, dim.height/2);
				return;
			}

        if(icon != null)
            icon.paint(g, inset, dim.height - icon.getSize().height - inset);

        int x0 = parent.getStartTextPosition();
        boolean isDir = isDir();
        int wt = dim.width-2*inset - x0 - (isDir ? arrow.getSize().width + 2 * inset : 0);
        FontMetrics fm = g.getFontMetrics();
        g.drawString(textForLength(wt, g), x0+inset, dim.height - inset - fm.getDescent());

        if(isDir)
            arrow.paint(g,
                        dim.width-arrow.getSize().width-inset,
                        1 + (dim.height - 2 * inset - arrow.getSize().height) / 2);
    }
    public String textForLength(int l, Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int st = fm.stringWidth(text);
        if(st <= l)
            return text;

        int pl = fm.stringWidth("..");
        if(pl >= l)
            return "..";

        int i=1;
        for(; i<text.length(); i++)
            if(fm.stringWidth(text.substring(0, i)) + pl >= l)
                break;

        return text.substring(0, i-1)+"..";
    }
}