/*
 *    author: Lloyd Dupont
 *    mail: lloyd@galador.net
 *    Copyright (C) 2000 Lloyd Dupont
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.applet.*;
import java.awt.*;
import java.util.*;

/**
 * controller of menu. You don't create them directly but, instead, create
 * MenuPanel and call it's <B>getController()</B> method.<BR>
 * it hold multiple value that each menu read
 * for its initialization. you should set the <B>applet</B> field in an
 * applet. <BR>
 * It could init most of its UI value from a Hashtable (you surely would to
 * use PList to init such Hashtable), with the given key:
 * <UL>
 * <LI> <B>bgcolor</B>: the background color as an <I>int</I> value
 * <LI> <B>textcolor</B>: the text color as an <I>int</I> value
 * <LI> <B>selcolor</B>: the selection (backgound) color as an <I>int</I> value
 * <LI> <B>bgimage</B>: the background image URL
 * <LI> <B>font</B>: the font name to use. beware that java know very few font.
 * <LI> <B>font-size</B>: the font size as an int.
 * <LI> <B>font-style</B> the font style as a string containing BOLD, ITALIC
 * separated by ' ', '|', '.'.
 * </UL>
 * number could be decimal int or '0x', '#' prefixed hexadecimal, or '0b'
 * prefixed binary int, or '0' prefixed octal int.
 *
 * @see #dispose
 * @see #init
 * @see MenuPanel
 */
public class MenuController
{
    public final MenuPanel root;
    public final DelayedAction handler = new DelayedAction();
    public final BackgroundAction todo = new BackgroundAction();

    // global info
    public Applet applet;

    // UI data
    public Color bgcolor;
    public Color textcolor;
    public Color selcolor = new Color(0xA0A0FF);
    public Image bgimage;
    public Font font;

    public MenuController(MenuPanel mp) {
        root = mp;
        if(mp == null)
            throw new NullPointerException();

        bgcolor = root.getBackground();
        textcolor = root.getForeground();
        font = root.getFont();
    }
	/**
	 * call this with proper Hashtable to initialize you controller.
	 */
    public void init(Hashtable keys) {
        if(keys == null)
            return;
        String val;

        val = (String) keys.get("bgcolor");
        if(val != null)
            bgcolor = new Color(parseInt(val));

        val = (String) keys.get("textcolor");
        if(val != null)
            textcolor = new Color(parseInt(val));

        val = (String) keys.get("selcolor");
        if(val != null)
            selcolor = new Color(parseInt(val));

        val = (String) keys.get("bgimage");
        if(val != null)
            bgimage = ImageCenter.loadImage(val);

        val = (String) keys.get("font");
        if(val == null)
            val = "";
        int style = Font.PLAIN;
        String sst = (String) keys.get("font-style");
        if(sst != null) {
            StringTokenizer st = new StringTokenizer(sst," \t\n\r|.");
            while(st.hasMoreTokens()) {
                sst = st.nextToken().toUpperCase();
                if(sst.equals("BOLD"))
                    style |= Font.BOLD;
                else if(sst.equals("ITALIC"))
                    style |= Font.ITALIC;
            }
        }
        String ssi = (String) keys.get("font-size");
        int size = 12;
        if(ssi != null)
            size = parseInt(ssi);
        if(val!=null || sst!=null || ssi!=null)
            font = new Font(val, style, size);

        root.setController(this);
    }
    int parseInt(String s) {
        if(s.startsWith("0x") || s.startsWith("0X") || s.startsWith("#") )
            return Integer.parseInt(s.substring(2), 16);
        if(s.startsWith("0b") || s.startsWith("0B") )
            return Integer.parseInt(s.substring(2), 2);
        else if(s.startsWith("0"))
            return Integer.parseInt(s.substring(2), 8);
        else
            return Integer.parseInt(s);
    }
	/**
	 * "free resource" by stopping inner thread
	 */
	public void dispose()
	{
		todo.dispose();
		handler.dispose();
	}
    protected void finalize()
        throws Throwable
    {
        dispose();
        super.finalize();
    }
}